]> Shamusworld >> Repos - rmac/blobdiff - rmac.c
Long forgotten helper script which is used to create tables for VS builds
[rmac] / rmac.c
diff --git a/rmac.c b/rmac.c
index 16af05ca6dfec465cf0a8b2feb9d975990041541..b6a7afe3745aca10df0829264af59157045149d9 100644 (file)
--- a/rmac.c
+++ b/rmac.c
@@ -1,7 +1,7 @@
 //
 // RMAC - Reboot's Macro Assembler for all Atari computers
 // RMAC.C - Main Application Code
-// Copyright (C) 199x Landon Dyer, 2011-2017 Reboot and Friends
+// Copyright (C) 199x Landon Dyer, 2011-2019 Reboot and Friends
 // RMAC derived from MADMAC v1.07 Written by Landon Dyer, 1986
 // Source utilised with the kind permission of Landon Dyer
 //
@@ -10,6 +10,7 @@
 #include "6502.h"
 #include "debug.h"
 #include "direct.h"
+#include "dsp56k.h"
 #include "error.h"
 #include "expr.h"
 #include "listing.h"
@@ -33,12 +34,14 @@ int glob_flag;                                      // Assume undefined symbols are global
 int lsym_flag;                                 // Include local symbols in object file
 int sbra_flag;                                 // Warn about possible short branches
 int prg_flag;                                  // !=0, produce .PRG executable (2=symbols)
+int prg_extend;                                        // !=0, output extended .PRG symbols
 int legacy_flag;                               // Do stuff like insert code in RISC assembler
 int obj_format;                                        // Object format flag
 int debug;                                             // [1..9] Enable debugging levels
 int err_flag;                                  // '-e' specified
 int err_fd;                                            // File to write error messages to
 int rgpu, rdsp;                                        // Assembling Jaguar GPU or DSP code
+int robjproc;                                  // Assembling Jaguar Object Processor code
 int dsp56001;                                  // Assembling DSP 56001 code
 int list_fd;                                   // File to write listing to
 int regbank;                                   // RISC register bank
@@ -54,6 +57,16 @@ int activecpu = CPU_68000;           // Active 68k CPU (68000 by default)
 int activefpu = FPU_NONE;              // Active FPU (none by default)
 
 
+//
+// Convert a string to uppercase
+//
+void strtoupper(char * s)
+{
+       while (*s)
+               *s++ &= 0xDF;
+}
+
+
 //
 // Manipulate file extension.
 //
@@ -136,10 +149,14 @@ void DisplayHelp(void)
                "                    a: ALCYON (use this for ST)\n"
                "                    b: BSD (use this for Jaguar)\n"
                "                    e: ELF\n"
+               "                    p: P56 (use this for DSP56001 only)\n"
+               "                    l: LOD (use this for DSP56001 only)\n"
                "                    x: com/exe/xex (Atari 800)\n"
                "  -i[path]          Directory to search for include files\n"
                "  -l[filename]      Create an output listing file\n"
                "  -l*[filename]     Create an output listing file without pagination\n"
+               "  -m[cpu]           Select default CPU. Available options:\n"
+               "                    68000, 68020, 68030, 68040, 68060, 6502, tom, jerry, 56001\n"
                "  -n                Don't do things behind your back in RISC assembler\n"
                "  -o file           Output file name\n"
                "  +o[value]         Turn a specific optimisation on\n"
@@ -150,11 +167,16 @@ void DisplayHelp(void)
                "                    o3: Outer displacement 0(an) to (an)        (off)\n"
                "                    o4: lea size(An),An to addq #size,An        (off)\n"
                "                    o5: Absolute long base displacement to word (off)\n"
+               "                    o6: Null branches to NOP                    (off)\n"
+               "                    o7: clr.l Dx to moveq #0,Dx                 (off)\n"
+               "                    o8: adda.w/l #x,Dy to addq.w/l #x,Dy        (off)\n"
+               "                    o9: adda.w/l #x,Dy to lea x(Dy),Dy          (off)\n"
                "  ~o[value]         Turn a specific optimisation off\n"
                "  +oall             Turn all optimisations on\n"
                "  ~oall             Turn all optimisations off\n"
                "  -p                Create an ST .prg (without symbols)\n"
                "  -ps               Create an ST .prg (with symbols)\n"
+               "  -px               Create an ST .prg (with exsymbols)\n"
                "                    Forces -fa\n"
                "  -r[size]          Pad segments to boundary size specified\n"
                "                    w: word (2 bytes, default alignment)\n"
@@ -183,7 +205,7 @@ void DisplayVersion(void)
                "| |  | | | | | | (_| | (__ \n"
                "|_|  |_| |_| |_|\\__,_|\\___|\n"
                "\nReboot's Macro Assembler\n"
-               "Copyright (C) 199x Landon Dyer, 2011-2017 Reboot\n"
+               "Copyright (C) 199x Landon Dyer, 2011-2019 Reboot\n"
                "V%01i.%01i.%01i %s (%s)\n\n", MAJOR, MINOR, PATCH, __DATE__, PLATFORM);
 }
 
@@ -238,32 +260,35 @@ int Process(int argc, char ** argv)
        char fnbuf[FNSIZ];                              // Filename buffer
        int i;                                                  // Iterator
 
-       errcnt = 0;                                             // Initialise error count
-       listing = 0;                                    // Initialise listing level
-       list_flag = 0;                                  // Initialise listing flag
-       verb_flag = perm_verb_flag;             // Initialise verbose flag
-       as68_flag = 0;                                  // Initialise as68 kludge mode
-       glob_flag = 0;                                  // Initialise .globl flag
-       sbra_flag = 0;                                  // Initialise short branch flag
-       debug = 0;                                              // Initialise debug flag
-       searchpath = NULL;                              // Initialise search path
-       objfname = NULL;                                // Initialise object filename
-       list_fname = NULL;                              // Initialise listing filename
-       err_fname = NULL;                               // Initialise error filename
-       obj_format = BSD;                               // Initialise object format
-       firstfname = NULL;                              // Initialise first filename
-       err_fd = ERROUT;                                // Initialise error file descriptor
-       err_flag = 0;                                   // Initialise error flag
-       rgpu = 0;                                               // Initialise GPU assembly flag
-       rdsp = 0;                                               // Initialise DSP assembly flag
+       errcnt = 0;                                             // Initialize error count
+       listing = 0;                                    // Initialize listing level
+       list_flag = 0;                                  // Initialize listing flag
+       verb_flag = perm_verb_flag;             // Initialize verbose flag
+       as68_flag = 0;                                  // Initialize as68 kludge mode
+       glob_flag = 0;                                  // Initialize .globl flag
+       sbra_flag = 0;                                  // Initialize short branch flag
+       debug = 0;                                              // Initialize debug flag
+       searchpath = NULL;                              // Initialize search path
+       objfname = NULL;                                // Initialize object filename
+       list_fname = NULL;                              // Initialize listing filename
+       err_fname = NULL;                               // Initialize error filename
+       obj_format = BSD;                               // Initialize object format
+       firstfname = NULL;                              // Initialize first filename
+       err_fd = ERROUT;                                // Initialize error file descriptor
+       err_flag = 0;                                   // Initialize error flag
+       rgpu = 0;                                               // Initialize GPU assembly flag
+       rdsp = 0;                                               // Initialize DSP assembly flag
+       robjproc = 0;                                   // Initialize OP assembly flag
        lsym_flag = 1;                                  // Include local symbols in object file
        regbank = BANK_N;                               // No RISC register bank specified
        orgactive = 0;                                  // Not in RISC org section
        orgwarning = 0;                                 // No ORG warning issued
-       segpadsize = 2;                                 // Initialise segment padding size
+       segpadsize = 2;                                 // Initialize segment padding size
+    dsp_orgmap[0].start = 0;           // Initialize 56001 org initial address
+    dsp_orgmap[0].memtype = ORG_P;     // Initialize 56001 org start segment
        m6502 = 0;                                              // 6502 mode off by default
 
-       // Initialise modules
+       // Initialize modules
        InitSymbolTable();                              // Symbol table
        InitTokenizer();                                // Tokenizer
        InitLineProcessor();                    // Line processor
@@ -308,7 +333,7 @@ int Process(int argc, char ** argv)
                                }
 
                                sy->sattr = DEFINED | EQUATED | ABS;
-                               sy->svalue = (*s ? (uint32_t)atoi(s) : 0);
+                               sy->svalue = (*s ? (uint64_t)atoi(s) : 0);
                                break;
                        case 'e':                               // Redirect error message output
                        case 'E':
@@ -331,6 +356,14 @@ int Process(int argc, char ** argv)
                                case 'E':
                                        obj_format = ELF;
                                        break;
+                case 'l':           // -fl = LOD
+                case 'L':
+                    obj_format = LOD;
+                    break;
+                case 'p':           // -fp = P56
+                case 'P':
+                    obj_format = P56;
+                                       break;
                                case 'x':                       // -fx = COM/EXE/XEX
                                case 'X':
                                        obj_format = XEX;
@@ -365,6 +398,37 @@ int Process(int argc, char ** argv)
                                list_flag = 1;
                                lnsave++;
                                break;
+                       case 'm':
+                       case 'M':
+                               if (strcmp(argv[argno] + 2, "68000") == 0)
+                                       d_68000();
+                               else if (strcmp(argv[argno] + 2, "68020") == 0)
+                                       d_68020();
+                               else if (strcmp(argv[argno] + 2, "68030") == 0)
+                                       d_68030();
+                               else if (strcmp(argv[argno] + 2, "68040") == 0)
+                                       d_68040();
+                               else if (strcmp(argv[argno] + 2, "68060") == 0)
+                                       d_68060();
+                               else if (strcmp(argv[argno] + 2, "68881") == 0)
+                                       d_68881();
+                               else if (strcmp(argv[argno] + 2, "68882") == 0)
+                                       d_68882();
+                               else if (strcmp(argv[argno] + 2, "56001") == 0)
+                                       d_56001();
+                               else if (strcmp(argv[argno] + 2, "6502") == 0)
+                                       d_6502();
+                               else if (strcmp(argv[argno] + 2, "tom") == 0)
+                                       d_gpu();
+                               else if (strcmp(argv[argno] + 2, "jerry") == 0)
+                                       d_dsp();
+                               else
+                               {
+                                       printf("Unrecognized CPU '%s'\n", argv[argno] + 2);
+                                       errcnt++;
+                                       return errcnt;
+                               }
+                               break;
                        case 'o':                               // Direct object file output
                        case 'O':
                                if (argv[argno][2] != EOS)
@@ -386,7 +450,8 @@ int Process(int argc, char ** argv)
                        case 'P':
                                /*
                                 * -p           .PRG generation w/o symbols
* -ps  .PRG generation with symbols
                                 */
                                switch (argv[argno][2])
                                {
@@ -399,6 +464,11 @@ int Process(int argc, char ** argv)
                                                prg_flag = 2;
                                                break;
 
+                                       case 'x':
+                                       case 'X':
+                                               prg_flag = 3;
+                                               break;
+
                                        default:
                                                printf("-p: syntax error\n");
                                                errcnt++;
@@ -524,6 +594,7 @@ int Process(int argc, char ** argv)
        //    -  "foo.o" for linkable output;
        //    -  "foo.prg" for GEMDOS executable (-p flag).
        SaveSection();
+       int temp_section = cursect;
 
        for(i=TEXT; i<=BSS; i<<=1)
        {
@@ -549,6 +620,20 @@ int Process(int argc, char ** argv)
                currentorg += 2;
        }
 
+       // This looks like an awful kludge...  !!! FIX !!!
+       if (temp_section & (M56001P | M56001X | M56001Y))
+       {
+               SwitchSection(temp_section);
+
+               if (chptr != dsp_currentorg->start)
+               {
+                       dsp_currentorg->end = chptr;
+                       dsp_currentorg++;
+               }
+       }
+
+       SwitchSection(TEXT);
+
        if (objfname == NULL)
        {
                if (firstfname == NULL)
@@ -571,7 +656,7 @@ int Process(int argc, char ** argv)
        if (errcnt == 0)
        {
                if ((fd = open(objfname, _OPEN_FLAGS, _PERM_MODE)) < 0)
-                       cantcreat(objfname);
+                       CantCreateFile(objfname);
 
                if (verb_flag)
                {
@@ -386,7 +450,8 @@ int Process(int argc, char ** argv)
                                /*
                                 * -p           .PRG generation w/o symbols
-                                * -ps  .PRG generation with symbols
+                                * -ps          .PRG generation with symbols
+                                * -px          .PRG generation with extended symbols
                                 */
                                switch (argv[argno][2])